﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Controls.Charts;
using Infragistics.Windows;
using System.Collections.Specialized;
using System.Collections;
using System.Reflection;

namespace CustomBindableInfoStrips
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ChartViewModel vm;
        public MainWindow()
        {
            InitializeComponent();

            vm = Root.Resources["vm"] as ChartViewModel;
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            vm.AddStrip();
        }

        private void button2_Click(object sender, RoutedEventArgs e)
        {
            vm.RemoveStrip();
        }
    }

    #region Custom Info Strip Behavior
    public partial class CustomVerticalInfoStripBehavior : DependencyObject
    {
        public static readonly DependencyProperty CustomVerticalInfoStripProperty =
            DependencyProperty.RegisterAttached("CustomVerticalInfoStrip",
                typeof(CustomVerticalInfoStrip),
                typeof(CustomVerticalInfoStripBehavior),
                new PropertyMetadata(null,
                    (o, e) =>
                    {
                        CustomVerticalInfoStripChanged(o as Axis,
                            e.OldValue as CustomVerticalInfoStrip,
                            e.NewValue as CustomVerticalInfoStrip);
                    }));

        public static CustomVerticalInfoStrip GetCustomVerticalInfoStrip(DependencyObject target)
        {
            return target.GetValue(CustomVerticalInfoStripProperty) as CustomVerticalInfoStrip;
        }

        public static void SetCustomVerticalInfoStrip(DependencyObject target, CustomVerticalInfoStrip value)
        {
            target.SetValue(CustomVerticalInfoStripProperty, value);
        }

        public static void CustomVerticalInfoStripChanged(
            Axis axis,
            CustomVerticalInfoStrip oldValue,
            CustomVerticalInfoStrip newValue)
        {
            if (axis == null)
                return;
            if (!(axis is NumericXAxis) && !(axis is CategoryXAxis) && !(axis is CategoryDateTimeXAxis))
                return;

            if (oldValue != null)
            {
                oldValue.DetachOwner(axis);
            }
            if (newValue != null)
            {
                newValue.AttachOwner(axis);
            }
        }
    }

    public class CustomVerticalInfoStrip : DependencyObject, INotifyPropertyChanged
    {
        #region Private Fields
        private Axis _owner;
        private XamDataChart _chart;
        private List<ContentControl> _infoStrips = new List<ContentControl>();
        private Canvas _container = new Canvas();
        #endregion

        #region Initializing Methods
        public void AttachOwner(Axis axis)
        {
            if (_owner != null)
            {
                DetachOwner(_owner);
            }

            _owner = axis;
            _chart = _owner.SeriesViewer as XamDataChart;

            if (_owner != null && _chart != null)
            {
                _chart.LayoutUpdated += _chart_LayoutUpdated;
               
                Panel.SetZIndex(_container, 100);
            }
        }     

        public void DetachOwner(Axis axis)
        {
            if (_owner != axis)
            {
                return;
            }

            _chart.LayoutUpdated -= _chart_LayoutUpdated;

            _owner = null;
            _chart = null;
        }
        #endregion

        #region Public Properties

        #region InfoStrip MemberPaths



        public string StripStartMemeberPath
        {
            get { return (string)GetValue(StripStartMemeberPathProperty); }
            set { SetValue(StripStartMemeberPathProperty, value); }
        }

        // Using a DependencyProperty as the backing store for StripStartMemeberPath.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StripStartMemeberPathProperty =
            DependencyProperty.Register("StripStartMemeberPath", typeof(string), typeof(CustomVerticalInfoStrip), new UIPropertyMetadata(null));



        public string StripEndMemberPath
        {
            get { return (string)GetValue(StripEndMemberPathProperty); }
            set { SetValue(StripEndMemberPathProperty, value); }
        }

        // Using a DependencyProperty as the backing store for StripEndMemberPath.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StripEndMemberPathProperty =
            DependencyProperty.Register("StripEndMemberPath", typeof(string), typeof(CustomVerticalInfoStrip), new UIPropertyMetadata(null));



        #endregion      

        #region ItemsSource
        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ItemsSource.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ItemsSourceProperty =
            DependencyProperty.Register(
                "ItemsSource",
                typeof(IEnumerable),
                typeof(CustomVerticalInfoStrip),
                new UIPropertyMetadata(null,
                    (o, e) =>
                    {
                        //clear controls
                        (o as CustomVerticalInfoStrip)._container.Children.Clear();
                        (o as CustomVerticalInfoStrip)._infoStrips.Clear();
                        //fill controls
                        foreach (var item in e.NewValue as IEnumerable)
                        {
                            ContentControl strip = new ContentControl
                                {
                                    Content = item,
                                    Background = Brushes.Red
                                };
                            strip.SetBinding(ContentControl.ContentTemplateProperty,
                                new Binding("StripItemTemplate") { Source = o });
                            (o as CustomVerticalInfoStrip)._container.Children.Add(strip);
                            (o as CustomVerticalInfoStrip)._infoStrips.Add(strip);    
                        }

                        if (e.NewValue != null && e.NewValue is INotifyCollectionChanged)
                        {
                            (e.NewValue as INotifyCollectionChanged).CollectionChanged +=
                                (ss, ee) =>
                                {
                                    if (ee.Action == NotifyCollectionChangedAction.Add)
                                    {
                                        ContentControl strip = new ContentControl
                                        {
                                            Content = ee.NewItems[0],
                                        };
                                        strip.SetBinding(ContentControl.ContentTemplateProperty,
                                            new Binding("StripItemTemplate") { Source = o });
                                        (o as CustomVerticalInfoStrip)._container.Children.Add(strip);
                                        (o as CustomVerticalInfoStrip)._infoStrips.Add(strip);    
                                    }

                                    if (ee.Action == NotifyCollectionChangedAction.Remove)
                                    {
                                        ContentControl stripToRemove = (o as CustomVerticalInfoStrip)._container.Children.Cast<ContentControl>().Where(c => c.Content == ee.OldItems[0]).FirstOrDefault();
                                        if (stripToRemove != null)
                                        {
                                            (o as CustomVerticalInfoStrip)._container.Children.Remove(stripToRemove);
                                            (o as CustomVerticalInfoStrip)._infoStrips.Remove(stripToRemove);    

                                        }
                                    }
                                };
                        }

                        if (e.OldValue != null && e.OldValue is INotifyCollectionChanged)
                        {
                            (e.OldValue as INotifyCollectionChanged).CollectionChanged -= (ss, ee) => { };
                        }
                    }));

     
        #endregion    

        #region Strip ItemTempalte


        public DataTemplate StripItemTemplate
        {
            get { return (DataTemplate)GetValue(StripItemTemplateProperty); }
            set { SetValue(StripItemTemplateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for StripItemTemplate.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StripItemTemplateProperty =
            DependencyProperty.Register(
                "StripItemTemplate",
                typeof(DataTemplate),
                typeof(CustomVerticalInfoStrip),
                new UIPropertyMetadata(null));


        #endregion

        #endregion

        #region Helper Methods

        private void RefreshVisualStrips()
        {
            if (_owner != null && ItemsSource != null )
            {
                var viewport = new Rect(0, 0, _owner.ActualWidth, _chart.ActualHeight);
                var window = _chart.WindowRect;
                ScalerParams sParams = new ScalerParams(window, viewport, _owner.IsInverted);
                
                foreach (ContentControl strip in _infoStrips)
                {
                    double unscaledStartValue = 0;
                    double unscaledEndValue = 0;

                    var pathStartValue = strip.Content.GetType().GetProperty(StripStartMemeberPath).GetValue(strip.Content, null);
                    var pathEndValue = strip.Content.GetType().GetProperty(StripEndMemberPath).GetValue(strip.Content, null);

                    if (pathStartValue != null && pathEndValue != null)
                    {
                        if (pathStartValue is DateTime && pathEndValue is DateTime)
                        {
                            unscaledStartValue = ((DateTime)pathStartValue).Ticks;
                            unscaledEndValue = ((DateTime)pathEndValue).Ticks;
                        }
                        if ((pathStartValue is int && pathEndValue is int) || (pathStartValue is double && pathEndValue is double))
                        {
                            double.TryParse(pathStartValue.ToString(), out unscaledStartValue);
                            double.TryParse(pathEndValue.ToString(), out unscaledEndValue);
                        }
                    }
             
                    double xStartCoordinate = _owner.GetScaledValue(unscaledStartValue, sParams);
                    double xEndCoordinate = _owner.GetScaledValue(unscaledEndValue, sParams);

                    strip.Height = _container.ActualHeight;
                    strip.Width = xEndCoordinate - xStartCoordinate;
                    Canvas.SetTop(strip, 0);
                    Canvas.SetLeft(strip, xStartCoordinate);
                    
                }
            } 
        }

        #endregion

        #region Event Handlers

        void _chart_LayoutUpdated(object sender, EventArgs e)
        {
            if (_container.Parent == null)
            {
                ContentPresenter presenter =
                    Utilities.GetDescendantFromName(_chart, "ContentPresenter") as ContentPresenter;

                if (presenter != null)
                {
                    Border chartContent =
                        Utilities.GetDescendantFromType(
                        presenter, typeof(Border), false) as Border;

                    if (chartContent != null)
                    {
                        (chartContent.Child as Grid).Children.Add(_container);
                    }
                }
            }
            RefreshVisualStrips();
        }


      

        #endregion

        #region INotifyPropertyChanged Memebers
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion
    }    
    #endregion


    #region Data
    public class ChartViewModel : PropertyChangedNotifier
    {
        private ObservableCollection<ChartData> chartSource;
        private ObservableCollection<InfoStripData> stripSource;

        public ChartViewModel()
        {
            ChartSource = ChartData.GenerateData(20);

            StripSource = new ObservableCollection<InfoStripData>();
            StripSource.Add(new InfoStripData
            {
                StartDate = DateTime.Now,
                EndDate = DateTime.Now.AddDays(2).AddHours(10),
                StartIndex = 0,
                EndIndex = 2,
                StartValue = 1.234,
                EndValue = 5.4,
                Data = "Some value",
                Color = Brushes.Red
            });

            StripSource.Add(new InfoStripData
            {
                StartDate = DateTime.Now.AddDays(5),
                EndDate = DateTime.Now.AddDays(6),
                StartIndex = 5,
                EndIndex = 8,
                StartValue = 7.7,
                EndValue = 12.5,
                Data = "Some other value",
                Color = Brushes.Green
            });

        }

        public ObservableCollection<ChartData> ChartSource
        {
            get { return chartSource; }
            set { chartSource = value; OnPropertyChagned("ChartSource"); }
        }

        public ObservableCollection<InfoStripData> StripSource
        {
            get { return stripSource; }
            set { stripSource = value; OnPropertyChagned("StripSource"); }
        }

        public void AddStrip()
        {
            List<Brush> brushes = new List<Brush>();
            foreach (PropertyInfo prop in typeof(Brushes).GetProperties())
            {
                brushes.Add((Brush)prop.GetValue(null, null));
            }

            Random r = new Random();
            int start = r.Next(17);
            int end = r.Next(start + 1, start + 3);
            StripSource.Add(new InfoStripData
            {
                StartDate = DateTime.Now.AddDays(start),
                EndDate = DateTime.Now.AddDays(end),
                StartIndex = start,
                EndIndex = end,
                StartValue = start + r.NextDouble(),
                EndValue = end + r.NextDouble(),
                Data = "New Value " + r.Next(1000),
                Color = brushes[r.Next(brushes.Count)]
            });

          
        }

        public void RemoveStrip()
        {
            StripSource.RemoveAt(0);
        }

    }

    public class ChartData
    {
        public DateTime Date { get; set; }
        public int Index { get; set; }
        public double XValue { get; set; }
        public double Value1 { get; set; }
        public double Value2 { get; set; }
        public double Value3 { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Index = 0,
                XValue = 0,
                Value1 = r.Next(20, 40),
                Value2 = r.Next(20, 40),
                Value3 = r.Next(20, 40),
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                    Index = i,
                    XValue = i + r.NextDouble()
                };

                if (r.NextDouble() > .5)
                {
                    dat.Value1 = data[i - 1].Value1 + r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 + r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 + r.NextDouble();
                }
                else
                {
                    dat.Value1 = data[i - 1].Value1 - r.NextDouble();
                    dat.Value2 = data[i - 1].Value2 - r.NextDouble();
                    dat.Value3 = data[i - 1].Value3 - r.NextDouble();
                }

                data.Add(dat);
            }

            return data;
        }
    }

    public class PropertyChangedNotifier : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    public class InfoStripData
    {
        public DateTime StartDate { get; set; }
        public DateTime EndDate { get; set; }

        public int StartIndex { get; set; }
        public int EndIndex { get; set; }

        public double StartValue { get; set; }
        public double EndValue { get; set; }

        public string Data { get; set; }
        public Brush Color { get; set; }
    }

    #endregion
}
